@@ -1,4 +1,4 @@
-/* $Id: cpl_image_resample_body.h,v 1.25 2011/07/20 11:20:40 llundin Exp $
+/* $Id: cpl_image_resample_body.h,v 1.25 2011-07-20 11:20:40 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -202,12 +202,13 @@ ADDTYPE(cpl_image_get_interpolated)(const CPL_TYPE * pixel,
     double ydel;
     double value, weight;
 
-    const cpl_size xfirst = (cpl_size)ceil (xpos - xradius);
-    const cpl_size xlast  = (cpl_size)floor(xpos + xradius);
-    const cpl_size yfirst = (cpl_size)ceil (ypos - yradius);
-    const cpl_size ylast  = (cpl_size)floor(ypos + yradius);
+    /* These image positions can become negative */
+    const intptr_t xfirst = (intptr_t)ceil (xpos - xradius);
+    const intptr_t xlast  = (intptr_t)floor(xpos + xradius);
+    const intptr_t yfirst = (intptr_t)ceil (ypos - yradius);
+    const intptr_t ylast  = (intptr_t)floor(ypos + yradius);
     cpl_flops skipped = 0;
-    cpl_size x, y;
+    intptr_t x, y;
 
 #if 0
     assert( pixel != NULL );
@@ -224,19 +225,19 @@ ADDTYPE(cpl_image_get_interpolated)(const CPL_TYPE * pixel,
     if (xfirst > xlast || yfirst > ylast) {
         /* For sufficiently small radii no pixels can be reached */
         *pconfid = 0;
-        return 0;
+        return 0.0;
     }
 
     /* Generate weights for all points in the Y-direction */
     for (y = yfirst; y <= ylast; y++)
-        yweight[y-yfirst] = pyprof[(cpl_size)(fabs((double)y - ypos)
-                                              * ytabsperpix+0.5)];
+        yweight[(size_t)(y-yfirst)] = pyprof[(size_t)(fabs((double)y - ypos)
+                                                      * ytabsperpix+0.5)];
 
     /* Iterate through all the points in the X-direction */
     for (x = xfirst; x <= xlast; x++) {
         xdel = (double)x - xpos;
         sqrest = sqyradius - xdel*xdel*sqyxratio;
-        xweight = pxprof[(int)(fabs(xdel) * xtabsperpix+0.5)];
+        xweight = pxprof[(size_t)(fabs(xdel) * xtabsperpix+0.5)];
 
         /* Iterate through all the points in the Y-direction */
         for (y = yfirst; y <= ylast; y++) {
@@ -250,22 +251,23 @@ ADDTYPE(cpl_image_get_interpolated)(const CPL_TYPE * pixel,
                (xradius, yradius) at (xpos, ypos) */
 
             /* The pixel weight is the product of the two 1D-profiles */
-            weight = xweight * yweight[y-yfirst];
+            weight = xweight * yweight[(size_t)(y-yfirst)];
 
             /* Sum of all absolute weights in the inclusion area */
             wconf += fabs(weight);
 
-            if (x < 1 || y < 1 || x > nx || y > ny) {
+            if (x < 1 || y < 1 || x > (intptr_t)nx || y > (intptr_t)ny) {
                 skipped += 5;
                 continue;
             }
 
-            if (bpm != NULL && bpm[(x-1) + (y-1) * nx] != CPL_BINARY_0) {
+            if (bpm != NULL &&
+                bpm[(size_t)((x-1) + (y-1) * nx)] != CPL_BINARY_0) {
                 skipped += 5;
                 continue;
             }
 
-            value = pixel[(x-1) + (y-1) * nx];
+            value = pixel[(size_t)((x-1) + (y-1) * nx)];
 
             /* Sum of weigths actually used */
             wsum += weight;
