@@ -539,7 +539,7 @@ double ADDTYPE(cpl_tools_get_variancesum)(const CPL_TYPE * a,
     for (i=0; i < n; i++) {
         const double delta = (double)a[i] - mean;
 
-        varsum += (double)i * delta * delta / (double)(i + 1);
+        varsum += (double)i * delta * (delta / (double)(i + 1));
         mean   += delta / (double)(i + 1);
     }
 
