@@ -1,4 +1,4 @@
-/* $Id: cpl_error-test.c,v 1.24 2010/11/12 10:02:01 llundin Exp $
+/* $Id: cpl_error-test.c,v 1.25 2013-04-26 08:44:58 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2010/11/12 10:02:01 $
- * $Revision: 1.24 $
- * $Name: cpl-6_3_1 $
+ * $Date: 2013-04-26 08:44:58 $
+ * $Revision: 1.25 $
+ * $Name: not supported by cvs2svn $
  */
 
 /*-----------------------------------------------------------------------------
@@ -397,6 +397,4 @@ static cpl_error_code cpl_error_test_ensure(cpl_error_code code)
 
     cpl_ensure_code(0, code);
 
-    return CPL_ERROR_NONE;
-
 }
