@@ -1,4 +1,4 @@
-/* $Id: cpl_image_basic-test.c,v 1.98.2.1 2013/04/22 06:52:07 llundin Exp $
+/* $Id: cpl_image_basic-test.c,v 1.103 2013-07-25 09:47:57 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2013/04/22 06:52:07 $
- * $Revision: 1.98.2.1 $
- * $Name: cpl-6_3_1 $
+ * $Date: 2013-07-25 09:47:57 $
+ * $Revision: 1.103 $
+ * $Name: not supported by cvs2svn $
  */
 
 /*-----------------------------------------------------------------------------
@@ -47,6 +47,7 @@
 #include "cpl_memory.h"
 #include "cpl_test.h"
 #include "cpl_math_const.h"
+#include "cpl_tools.h"
 
 /*-----------------------------------------------------------------------------
                                    Defines
@@ -71,6 +72,9 @@ static void cpl_image_complex_mult(size_t o1, size_t o2, size_t n);
 static void cpl_image_complex_mult_d(size_t o1, size_t o2, size_t n);
 static void cpl_image_get_fwhm_test(void);
 static void cpl_image_divide_test_one(void);
+static void cpl_image_power_test(void);
+static void cpl_image_hypot_test(void);
+static void cpl_image_bitwise_test(cpl_size, cpl_size);
 
 /*-----------------------------------------------------------------------------
                                   Main
@@ -110,6 +114,7 @@ int main(void)
     int                 is_bad;
     int                 i, j, k;
     cpl_error_code      error;
+    cpl_boolean         do_bench;
 
     /*
      * FIXME: Ideally, we should simply use the macro I 
@@ -130,8 +135,27 @@ int main(void)
 
     cpl_test_init(PACKAGE_BUGREPORT, CPL_MSG_WARNING);
 
+    do_bench = cpl_msg_get_level() <= CPL_MSG_INFO ? CPL_TRUE : CPL_FALSE;
+
     /* Insert tests below */
 
+    cpl_image_bitwise_test(IMAGESZ, IMAGESZ);
+    cpl_image_bitwise_test(1, 1);
+    cpl_image_bitwise_test(17, 15);
+    cpl_image_bitwise_test(3, 32);
+    if (do_bench) {
+        double secs = cpl_test_get_walltime();
+        for (i = 0; i < 2; i++) {
+            cpl_image_bitwise_test(IMAGESZ/2, IMAGESZ/2);
+            cpl_image_bitwise_test(IMAGESZ,   IMAGESZ);
+            cpl_image_bitwise_test(IMAGESZ*2, IMAGESZ*2);
+            cpl_image_bitwise_test(IMAGESZ*4, IMAGESZ*4);
+        }
+        secs = cpl_test_get_walltime() - secs;
+        cpl_msg_info(cpl_func, "Time to test bit-wise calls [s]: %g", secs);
+    }
+    cpl_image_hypot_test();
+    cpl_image_power_test();
     cpl_image_divide_test_one();
 
     cpl_image_get_fwhm_test();
@@ -1009,6 +1033,10 @@ static void cpl_image_complex_mult(size_t o1, size_t o2, size_t n)
             else if (!isfinite(i1) || !isfinite(i2))
                 cpl_test_eq(isinf(i1), isinf(i2));
             else {
+                /* this test will fail if SSE is enabled but mfpmath=387, as
+                 * the comparison numbers are then generated with different
+                 * precision, the relative difference stays below ~ 1e-6,
+                 * also set mfpmath=sse for consistent results */
                 cpl_test_abs(i1, i2, FLT_EPSILON);
                 cpl_test_abs(r1, r2, FLT_EPSILON);
             }
@@ -1101,6 +1129,10 @@ static void cpl_image_complex_mult_d(size_t o1, size_t o2, size_t n)
             else if (!isfinite(i1) || !isfinite(i2))
                 cpl_test_eq(isinf(i1), isinf(i2));
             else {
+                /* this test will fail if SSE is enabled but mfpmath=387, as
+                 * the comparison numbers are then generated with different
+                 * precision, the relative difference stays below ~ 1e-15,
+                 * also set mfpmath=sse for consistent results */
                 cpl_test_abs(i1, i2, DBL_EPSILON);
                 cpl_test_abs(r1, r2, DBL_EPSILON);
             }
@@ -1787,3 +1819,363 @@ static void cpl_image_divide_test_one(void) {
     cpl_image_delete(img2);
 
 }
+
+/*----------------------------------------------------------------------------*/
+/**
+  @internal
+  @brief  Test the CPL function
+  @return void
+  @see cpl_image_power()
+
+ */
+/*----------------------------------------------------------------------------*/
+static void cpl_image_hypot_test(void) {
+
+    const cpl_type htype[2] = {CPL_TYPE_FLOAT, CPL_TYPE_DOUBLE};
+
+    for (int i1type = 0; i1type < 2; i1type++) {
+        for (int i2type = 0; i2type < 2; i2type++) {
+            for (int i3type = 0; i3type < 2; i3type++) {
+                const double tol =
+                    htype[i1type] == CPL_TYPE_FLOAT ||
+                    htype[i2type] == CPL_TYPE_FLOAT ||
+                    htype[i3type] == CPL_TYPE_FLOAT
+                    ? 40.0 * FLT_EPSILON
+                    : 40.0 * DBL_EPSILON;
+
+                cpl_image * img1 = cpl_image_new(IMAGESZ, IMAGESZ, htype[i1type]);
+                cpl_image * img2 = cpl_image_new(IMAGESZ, IMAGESZ, htype[i2type]);
+                cpl_image * img3 = cpl_image_new(IMAGESZ, IMAGESZ, htype[i3type]);
+                cpl_image * img1copy;
+                cpl_image * img2copy;
+                cpl_error_code code;
+
+                code = cpl_image_fill_noise_uniform(img1, 1.0, MAGIC_PIXEL);
+                cpl_test_eq_error(code, CPL_ERROR_NONE);
+                img1copy = cpl_image_duplicate(img1);
+
+                code = cpl_image_fill_noise_uniform(img2, 1.0, MAGIC_PIXEL);
+                cpl_test_eq_error(code, CPL_ERROR_NONE);
+                img2copy = cpl_image_duplicate(img2);
+
+                code = cpl_image_hypot(img3, img1, img2);
+                cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+                code = cpl_image_power(img1, 2.0);
+                cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+                code = cpl_image_power(img2, 2.0);
+                cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+                code = cpl_image_add(img1, img2);
+                cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+                code = cpl_image_power(img1, 0.5);
+                cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+                cpl_test_image_abs(img1, img3, tol);
+
+                code = cpl_image_hypot(img1copy, NULL, img2copy);
+                cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+                /* The in-place is not as accurate (w. all float data) ... */
+                cpl_test_image_abs(img1copy, img3, 1.5 * tol);
+
+                cpl_image_delete(img1);
+                cpl_image_delete(img2);
+                cpl_image_delete(img3);
+
+                /* Verify handling of incompatible size */
+                img3 = cpl_image_new(IMAGESZ+1, IMAGESZ, htype[i1type]);
+
+                code = cpl_image_hypot(img3, img1copy, img2copy);
+                cpl_test_eq_error(code, CPL_ERROR_INCOMPATIBLE_INPUT);
+
+                code = cpl_image_hypot(img1copy, img3, img2copy);
+                cpl_test_eq_error(code, CPL_ERROR_INCOMPATIBLE_INPUT);
+
+
+                code = cpl_image_hypot(img1copy, img2copy, img3);
+                cpl_test_eq_error(code, CPL_ERROR_INCOMPATIBLE_INPUT);
+
+                code = cpl_image_hypot(img3, NULL, img2copy);
+                cpl_test_eq_error(code, CPL_ERROR_INCOMPATIBLE_INPUT);
+
+                code = cpl_image_hypot(img2copy, NULL, img3);
+                cpl_test_eq_error(code, CPL_ERROR_INCOMPATIBLE_INPUT);
+
+                cpl_image_delete(img3);
+
+                /* Verify handling of unsupported type */
+                img3 = cpl_image_new(IMAGESZ+1, IMAGESZ, htype[i1type]
+                                     | CPL_TYPE_COMPLEX);
+
+                code = cpl_image_hypot(img3, img1copy, img2copy);
+                cpl_test_eq_error(code, CPL_ERROR_INVALID_TYPE);
+
+                code = cpl_image_hypot(img1copy, img3, img2copy);
+                cpl_test_eq_error(code, CPL_ERROR_INVALID_TYPE);
+
+
+                code = cpl_image_hypot(img1copy, img2copy, img3);
+                cpl_test_eq_error(code, CPL_ERROR_INVALID_TYPE);
+
+                code = cpl_image_hypot(img3, NULL, img2copy);
+                cpl_test_eq_error(code, CPL_ERROR_INVALID_TYPE);
+
+                code = cpl_image_hypot(img2copy, NULL, img3);
+                cpl_test_eq_error(code, CPL_ERROR_INVALID_TYPE);
+
+                cpl_image_delete(img3);
+
+                /* Verify handling of NULL pointers */
+                code = cpl_image_hypot(img1copy, img2copy, NULL);
+                cpl_test_eq_error(code, CPL_ERROR_NULL_INPUT);
+
+                code = cpl_image_hypot(NULL, img1copy, img2copy);
+                cpl_test_eq_error(code, CPL_ERROR_NULL_INPUT);
+
+                cpl_image_delete(img1copy);
+                cpl_image_delete(img2copy);
+            }
+        }
+    }
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+  @internal
+  @brief  Test the CPL function
+  @return void
+  @see cpl_image_power()
+
+ */
+/*----------------------------------------------------------------------------*/
+static void cpl_image_power_test(void) {
+
+    cpl_image * img1 = cpl_image_new(IMAGESZ, IMAGESZ, CPL_TYPE_DOUBLE);
+    cpl_image * img2;
+    cpl_error_code code;
+
+    /* Avoid division by zero for now */
+    code = cpl_image_fill_noise_uniform(img1, 1.0, MAGIC_PIXEL);
+    cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+    img2 = cpl_image_power_create(img1, 1.0);
+    cpl_test_error(CPL_ERROR_NONE);
+    cpl_test_nonnull(img2);
+
+    code = cpl_image_power(img1, -2.0);
+    cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+    code = cpl_image_power(img1, -0.5);
+    cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+    cpl_test_image_abs(img1, img2, 40.0 * DBL_EPSILON);
+
+    cpl_image_delete(img2);
+    img2 = cpl_image_power_create(img1, 1.0);
+    cpl_test_error(CPL_ERROR_NONE);
+    cpl_test_nonnull(img2);
+
+    code = cpl_image_power(img1, -3.0);
+    cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+    code = cpl_image_power(img1, -1.0/3.0);
+    cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+    cpl_test_image_abs(img1, img2, 100.0 * DBL_EPSILON);
+
+    cpl_image_delete(img1);
+    cpl_image_delete(img2);
+}
+
+
+/*----------------------------------------------------------------------------*/
+/**
+  @internal
+  @brief  Test the CPL function
+  @return void
+  @see cpl_image_and(), cpl_image_or(), cpl_image_xor()
+
+ */
+/*----------------------------------------------------------------------------*/
+static void cpl_image_bitwise_test(cpl_size nx, cpl_size ny)
+{
+
+    cpl_error_code (*fimage[3])(cpl_image *, const cpl_image *,
+                               const cpl_image *)
+        = {cpl_image_and, cpl_image_or, cpl_image_xor};
+    cpl_error_code (*fscalar[3])(cpl_image *, const cpl_image *,
+                                cpl_bitmask)
+        = {cpl_image_and_scalar, cpl_image_or_scalar, cpl_image_xor_scalar};
+    const size_t nfunc = sizeof(fimage)/sizeof(*fimage);
+    
+    cpl_image * imf   = cpl_image_new(nx,     ny,     CPL_TYPE_FLOAT);
+    cpl_image * im01  = cpl_image_new(nx,     ny + 1, CPL_TYPE_INT);
+    cpl_image * im10  = cpl_image_new(nx + 1, ny,     CPL_TYPE_INT);
+    cpl_image * im0   = cpl_image_new(nx,     ny,     CPL_TYPE_INT);
+    cpl_image * im1   = cpl_image_new(nx,     ny,     CPL_TYPE_INT);
+    cpl_image * im2   = cpl_image_new(nx,     ny,     CPL_TYPE_INT);
+    cpl_mask *  mask0 = cpl_mask_new(nx,      ny);
+    cpl_error_code code;
+
+    cpl_test_eq( sizeof(fimage), sizeof(fscalar));
+
+    /* Test the handling of the 3 possible error types for all 7 functions */
+    for (size_t i = 0; i < nfunc; i++) {
+        const cpl_image * im3 = im1;
+        for (size_t i2 = 0; i2 < 2; i2++, im3 = NULL) {
+            code = fimage[i](NULL, im3, im2);
+            cpl_test_eq_error(code, CPL_ERROR_NULL_INPUT);
+            code = fimage[i](im1, im3, NULL);
+            cpl_test_eq_error(code, CPL_ERROR_NULL_INPUT);
+            code = fscalar[i](NULL, im3, 0);
+            cpl_test_eq_error(code, CPL_ERROR_NULL_INPUT);
+
+            code = fimage[i](im01, im3, im2);
+            cpl_test_eq_error(code, CPL_ERROR_INCOMPATIBLE_INPUT);
+            code = fimage[i](im10, im3, im2);
+            cpl_test_eq_error(code, CPL_ERROR_INCOMPATIBLE_INPUT);
+            code = fimage[i](im1, im3, im01);
+            cpl_test_eq_error(code, CPL_ERROR_INCOMPATIBLE_INPUT);
+            code = fimage[i](im1, im3, im10);
+            cpl_test_eq_error(code, CPL_ERROR_INCOMPATIBLE_INPUT);
+
+            code = fimage[i](imf, im3, im2);
+            cpl_test_eq_error(code, CPL_ERROR_INVALID_TYPE);
+            code = fimage[i](im1, im3, imf);
+            cpl_test_eq_error(code, CPL_ERROR_INVALID_TYPE);
+            code = fscalar[i](imf, im3, 0);
+            cpl_test_eq_error(code, CPL_ERROR_INVALID_TYPE);
+
+            if (i == 0) {
+                code = cpl_image_not(NULL, im3);
+                cpl_test_eq_error(code, CPL_ERROR_NULL_INPUT);
+                code = cpl_image_not(imf, im3);
+                cpl_test_eq_error(code, CPL_ERROR_INVALID_TYPE);
+            }
+        }
+
+        code = fimage[i](im1, im01, im2);
+        cpl_test_eq_error(code, CPL_ERROR_INCOMPATIBLE_INPUT);
+        code = fimage[i](im1, im10, im2);
+        cpl_test_eq_error(code, CPL_ERROR_INCOMPATIBLE_INPUT);
+        code = fscalar[i](im1, im01, 0);
+        cpl_test_eq_error(code, CPL_ERROR_INCOMPATIBLE_INPUT);
+        code = fscalar[i](im1, im10, 0);
+        cpl_test_eq_error(code, CPL_ERROR_INCOMPATIBLE_INPUT);
+        code = fscalar[i](im01, im1, 0);
+        cpl_test_eq_error(code, CPL_ERROR_INCOMPATIBLE_INPUT);
+        code = fscalar[i](im10, im1, 0);
+        cpl_test_eq_error(code, CPL_ERROR_INCOMPATIBLE_INPUT);
+
+        code = fimage[i](im1, imf, im2);
+        cpl_test_eq_error(code, CPL_ERROR_INVALID_TYPE);
+        code = fscalar[i](im1, imf, 0);
+        cpl_test_eq_error(code, CPL_ERROR_INVALID_TYPE);
+    }
+
+    code = cpl_image_not(im1, im01);
+    cpl_test_eq_error(code, CPL_ERROR_INCOMPATIBLE_INPUT);
+    code = cpl_image_not(im1, im10);
+    cpl_test_eq_error(code, CPL_ERROR_INCOMPATIBLE_INPUT);
+
+    code = cpl_image_fill_noise_uniform(im1,
+                                        -cpl_tools_ipow(2.0, 31),
+                                        cpl_tools_ipow(2.0, 31) - 1.0);
+    cpl_test_eq_error(code, CPL_ERROR_NONE);
+    code = cpl_image_not(im2, im1);
+    cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+    code = cpl_image_or(im0, im1, im2);
+    cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+    code = cpl_mask_threshold_image(mask0, im0, -1.5, -0.5, CPL_BINARY_0);
+    cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+    cpl_test_zero(cpl_mask_count(mask0));
+
+    code = cpl_image_xor(im0, im1, im2);
+    cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+    code = cpl_mask_threshold_image(mask0, im0, -1.5, -0.5, CPL_BINARY_0);
+    cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+    cpl_test_zero(cpl_mask_count(mask0));
+
+    code = cpl_image_and(im0, im1, im2);
+    cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+    code = cpl_mask_threshold_image(mask0, im0, -0.5, 0.5, CPL_BINARY_0);
+    cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+    cpl_test_zero(cpl_mask_count(mask0));
+
+    code = cpl_image_and_scalar(im0, im1, 15);
+    cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+    code = cpl_mask_threshold_image(mask0, im0, -0.5, 15.5, CPL_BINARY_0);
+    cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+    cpl_test_zero(cpl_mask_count(mask0));
+    if (cpl_mask_count(mask0)) {
+        int idummy;
+        cpl_test_eq(cpl_image_get(im0, 1, 1, &idummy), 15);
+    }
+
+    code = cpl_image_and_scalar(im0, im1, 0);
+    cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+    code = cpl_mask_threshold_image(mask0, im0, -0.5, 0.5, CPL_BINARY_0);
+    cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+    cpl_test_zero(cpl_mask_count(mask0));
+
+    code = cpl_image_and_scalar(im0, im1, 15);
+    cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+    code = cpl_mask_threshold_image(mask0, im0, -0.5, 15.5, CPL_BINARY_0);
+    cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+    cpl_test_zero(cpl_mask_count(mask0));
+
+    code = cpl_image_and_scalar(im0, im1, (cpl_bitmask)-1);
+    cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+    cpl_test_image_abs(im0, im1, 0.0);
+
+    code = cpl_image_or_scalar(im0, im1, 0);
+    cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+    cpl_test_image_abs(im0, im1, 0.0);
+
+    code = cpl_image_xor_scalar(im0, im1, 0);
+    cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+    cpl_test_image_abs(im0, im1, 0.0);
+
+    code = cpl_image_or_scalar(im0, im1, (cpl_bitmask)-1);
+    cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+    code = cpl_mask_threshold_image(mask0, im0, -1.5, -0.5, CPL_BINARY_0);
+    cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+    cpl_test_zero(cpl_mask_count(mask0));
+
+    code = cpl_image_xor_scalar(im0, im1, (cpl_bitmask)-1);
+    cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+    code = cpl_image_not(im0, NULL);
+    cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+    cpl_test_image_abs(im0, im1, 0.0);
+
+    cpl_image_delete(imf);
+    cpl_image_delete(im01);
+    cpl_image_delete(im10);
+    cpl_image_delete(im0);
+    cpl_image_delete(im1);
+    cpl_image_delete(im2);
+    cpl_mask_delete(mask0);
+}
