@@ -1,4 +1,4 @@
-/* $Id: cpl_test-test.c,v 1.32 2012/08/23 12:52:49 llundin Exp $
+/* $Id: cpl_test-test.c,v 1.35 2013-04-08 07:55:52 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2012/08/23 12:52:49 $
- * $Revision: 1.32 $
- * $Name: cpl-6_3_1 $
+ * $Date: 2013-04-08 07:55:52 $
+ * $Revision: 1.35 $
+ * $Name: not supported by cvs2svn $
  */
 
 /*-----------------------------------------------------------------------------
@@ -123,6 +123,7 @@ int main(void)
     cpl_array      * arr2s;
     cpl_image      * img1;
     cpl_image      * img2;
+    cpl_image      * img3;
     cpl_imagelist  * imglist1;
     cpl_imagelist  * imglist2;
     cpl_polynomial * poly1;
@@ -131,6 +132,7 @@ int main(void)
     cpl_mask       * mask2;
     cpl_errorstate   cleanstate;
     cpl_errorstate   errstate;
+    cpl_error_code   error;
     double           wallstart, cpustart;
     double           wallstop,  cpustop;
     cpl_size         tested, failed;
@@ -216,11 +218,13 @@ int main(void)
     cpl_test_lt(0.0,  1.0);
 
     vec1 = cpl_vector_new(IMAGESZ);
-    cpl_vector_fill(vec1, 0.0);
+    error = cpl_vector_fill(vec1, 0.0);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
     vec2 = cpl_vector_duplicate(vec1);
     cpl_test_vector_abs(vec1, vec2, 0.0);
 
-    cpl_vector_add_scalar(vec1, 1.0);
+    error = cpl_vector_add_scalar(vec1, 1.0);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
 
     cpl_test_vector_abs(vec1, vec2, 1.0 + FLT_EPSILON); /* Round-off */
 
@@ -228,40 +232,48 @@ int main(void)
                 cpl_test_get_bytes_vector(vec2));
 
     mat1 = cpl_matrix_new(IMAGESZ, IMAGESZ);
-    cpl_matrix_fill(mat1, 0.0);
+    error = cpl_matrix_fill(mat1, 0.0);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
     mat2 = cpl_matrix_duplicate(mat1);
     cpl_test_matrix_abs(mat1, mat2, 0.0);
 
-    cpl_matrix_add_scalar(mat1, 1.0);
+    error = cpl_matrix_add_scalar(mat1, 1.0);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
 
     cpl_test_matrix_abs(mat1, mat2, 1.0 + FLT_EPSILON); /* Round-off */
 
     arr1i = cpl_array_new(IMAGESZ, CPL_TYPE_INT);
-    cpl_array_fill_window(arr1i, 0, IMAGESZ, 0.0);
+    error = cpl_array_fill_window(arr1i, 0, IMAGESZ, 0.0);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
     arr2i = cpl_array_duplicate(arr1i);
     cpl_test_array_abs(arr1i, arr2i, 0.0);
 
-    cpl_array_add_scalar(arr1i, 1.0);
+    error = cpl_array_add_scalar(arr1i, 1.0);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
 
     cpl_test_array_abs(arr1i, arr2i, 1.0 + FLT_EPSILON); /* Round-off */
 
 
     arr1f = cpl_array_new(IMAGESZ, CPL_TYPE_FLOAT);
-    cpl_array_fill_window(arr1f, 0, IMAGESZ, 0.0);
+    error = cpl_array_fill_window(arr1f, 0, IMAGESZ, 0.0);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
     arr2f = cpl_array_duplicate(arr1f);
     cpl_test_array_abs(arr1f, arr2f, 0.0);
 
-    cpl_array_add_scalar(arr1f, 1.0);
+    error = cpl_array_add_scalar(arr1f, 1.0);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
 
     cpl_test_array_abs(arr1f, arr2f, 1.0 + FLT_EPSILON); /* Round-off */
 
 
     arr1d = cpl_array_new(IMAGESZ, CPL_TYPE_DOUBLE);
-    cpl_array_fill_window(arr1d, 0, IMAGESZ, 0.0);
+    error = cpl_array_fill_window(arr1d, 0, IMAGESZ, 0.0);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
     arr2d = cpl_array_duplicate(arr1d);
     cpl_test_array_abs(arr1d, arr2d, 0.0);
 
-    cpl_array_add_scalar(arr1d, 1.0);
+    error = cpl_array_add_scalar(arr1d, 1.0);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
 
     cpl_test_array_abs(arr1d, arr2d, 1.0 + FLT_EPSILON); /* Round-off */
 
@@ -282,8 +294,17 @@ int main(void)
     img2 = cpl_image_duplicate(img1);
 
     cpl_test_image_abs(img1, img2, 0.0);
+    cpl_test_image_rel(img1, img2, 0.0);
 
-    cpl_image_add_scalar(img1, 1.0);
+    error = cpl_image_multiply_scalar(img1, 2.0);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+    cpl_test_image_rel(img1, img2, 1.0);
+
+    error = cpl_image_multiply_scalar(img1, 0.5);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+
+    error = cpl_image_add_scalar(img1, 1.0);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
 
     cpl_test_image_abs(img1, img2, 1.0 + FLT_EPSILON); /* Round-off */
 
@@ -295,7 +316,8 @@ int main(void)
     cpl_imagelist_set(imglist1, img2, 1);
     imglist2 = cpl_imagelist_duplicate(imglist1);
 
-    cpl_imagelist_add_scalar(imglist2, 1.0);
+    error = cpl_imagelist_add_scalar(imglist2, 1.0);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
 
     cpl_test_imagelist_abs(imglist1, imglist2, 1.0 + FLT_EPSILON);
 
@@ -309,7 +331,8 @@ int main(void)
 
     mask1 = cpl_mask_new(IMAGESZ, IMAGESZ);
     mask2 = cpl_mask_new(IMAGESZ, IMAGESZ);
-    cpl_mask_not(mask2);
+    error = cpl_mask_not(mask2);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
 
     cpl_test_eq_mask(mask1, mask1);
     cpl_test_eq_mask(mask2, mask2);
@@ -415,10 +438,23 @@ int main(void)
 
     cpl_test_array_abs(arr1s, arr2s, 0.0); nxfail++;
 
-    cpl_test_image_abs(NULL, NULL, 1.0); nxfail++;
+    cpl_test_image_abs(img1, NULL, 1.0); nxfail++;
+    cpl_test_image_abs(NULL, img2, 1.0); nxfail++;
+    cpl_test_image_rel(img1, NULL, 1.0); nxfail++;
+    cpl_test_image_rel(NULL, img2, 1.0); nxfail++;
 
     cpl_test_image_abs(img1, img2, 0.5); nxfail++;
 
+    cpl_test_image_rel(img1, img1, -0.5); nxfail++;
+
+    img3 = cpl_image_duplicate(img1);
+
+    error = cpl_image_multiply_scalar(img1, 2.0);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+    cpl_test_image_rel(img1, img3, 1.0 - FLT_EPSILON); nxfail++;
+    error = cpl_image_multiply_scalar(img1, 0.5);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+
     cpl_test_imagelist_abs(NULL,     NULL,     1.0); nxfail++;
     cpl_test_imagelist_abs(imglist1, NULL,     1.0); nxfail++;
     cpl_test_imagelist_abs(NULL,     imglist2, 1.0); nxfail++;
@@ -498,6 +534,7 @@ int main(void)
     cpl_array_delete(arr2s);
     cpl_image_delete(img1);
     cpl_image_delete(img2);
+    cpl_image_delete(img3);
     (void)cpl_imagelist_unset(imglist1, 0);
     cpl_imagelist_delete(imglist1);
     cpl_imagelist_delete(imglist2);
