@@ -26,9 +26,20 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#  include <config.h>
 #endif
 
+#include <string.h>
+#include <assert.h>
+#include <unistd.h>
+
+#ifdef HAVE_OPENSSL_EVP_H
+#  include <openssl/evp.h>
+#endif
+
+#include <fitsio.h>
+#include <fitsio2.h>
+
 #include <cpl_propertylist.h>
 #include <cpl_msg.h>
 #include <cpl_memory.h>
@@ -38,12 +49,7 @@
 
 #include "cpl_dfs.h"
 
-#include <fitsio2.h>
 
-#include <string.h>
-#include <fitsio.h>
-#include <assert.h>
-#include <unistd.h>
 
 /**
  * @defgroup cpl_dfs       DFS related functions
@@ -58,9 +64,12 @@
  * Product keywords aliases
  */
 
+#define DATE                        "DATE"
 #define DATAMD5                     "DATAMD5"
+#define ARCFILE                     "ARCFILE"
 #define PIPEFILE                    "PIPEFILE"
 #define CHECKSUM                    "CHECKSUM"
+#define DATASUM                     "DATASUM"
 #define PRO_DID                     "ESO PRO DID"
 #define DPR_CATG                    "ESO DPR CATG"
 #define PRO_CATG                    CPL_DFS_PRO_CATG 
@@ -68,18 +77,19 @@
 #define DPR_TECH                    "ESO DPR TECH"
 #define PRO_TECH                    CPL_DFS_PRO_TECH
 #define PRO_SCIENCE                 CPL_DFS_PRO_SCIENCE
+#define PRO_ANCESTOR                "ESO PRO ANCESTOR"
 #define PRO_DATE                    "ESO PRO DATE"
 #define PRO_DATANCOM                "ESO PRO DATANCOM"
-#define PRO_REC_ID                  "ESO PRO REC1 ID"
-#define PRO_REC_DRS_ID              "ESO PRO REC1 DRS ID"
-#define PRO_REC_PIPE_ID             "ESO PRO REC1 PIPE ID"
-#define PRO_REC_RAWi_NAME           "ESO PRO REC1 RAW%d NAME"
-#define PRO_REC_RAWi_CATG           "ESO PRO REC1 RAW%d CATG"
-#define PRO_REC_CALi_NAME           "ESO PRO REC1 CAL%d NAME"
-#define PRO_REC_CALi_CATG           "ESO PRO REC1 CAL%d CATG"
-#define PRO_REC_CALi_DATAMD5        "ESO PRO REC1 CAL%d " DATAMD5
-#define PRO_REC_PARAMi_NAME         "ESO PRO REC1 PARAM%d NAME"
-#define PRO_REC_PARAMi_VALUE        "ESO PRO REC1 PARAM%d VALUE"
+#define PRO_RECi_ID                 "ESO PRO REC%d ID"
+#define PRO_RECi_DRS_ID             "ESO PRO REC%d DRS ID"
+#define PRO_RECi_PIPE_ID            "ESO PRO REC%d PIPE ID"
+#define PRO_RECi_RAWi_NAME          "ESO PRO REC%d RAW%d NAME"
+#define PRO_RECi_RAWi_CATG          "ESO PRO REC%d RAW%d CATG"
+#define PRO_RECi_CALi_NAME          "ESO PRO REC%d CAL%d NAME"
+#define PRO_RECi_CALi_CATG          "ESO PRO REC%d CAL%d CATG"
+#define PRO_RECi_CALi_DATAMD5       "ESO PRO REC%d CAL%d " DATAMD5
+#define PRO_RECi_PARAMi_NAME        "ESO PRO REC%d PARAM%d NAME"
+#define PRO_RECi_PARAMi_VALUE       "ESO PRO REC%d PARAM%d VALUE"
 
 #define MAX_PLENGTH (64)
 
@@ -101,7 +111,7 @@
 #define CPL_OFF_TYPE OFF_T
 #endif
 
-#define CPL_DFS_PRO_DID "PRO-1.15"
+#define CPL_DFS_PRO_DID "PRO-1.16"
 
 
 /*-----------------------------------------------------------------------------
@@ -125,6 +135,9 @@
 static char * cpl_dfs_extract_printable(const char *) CPL_ATTR_ALLOC;
 #endif
 
+inline static cpl_error_code _cpl_dfs_sign_product(const cpl_frame *frame,
+                                                   cxuint flags);
+
 static cpl_error_code cpl_dfs_update_product_header_(cpl_frameset *);
 
 static char * cpl_dfs_find_md5sum(fitsfile *) CPL_ATTR_ALLOC;
@@ -556,6 +569,8 @@ cpl_error_code cpl_dfs_setup_product_header(cpl_propertylist *header,
     cpl_frameset_iterator *it = NULL;
     cpl_propertylist *plist;
     const cpl_parameter *param;
+    int               level = 0;
+    int               empty = 0;
     int               nraw;
     int               ncal;
     int               npar;
@@ -757,7 +772,8 @@ cpl_error_code cpl_dfs_setup_product_header(cpl_propertylist *header,
         if (first_frame == inherit_frame) {
             plist = cpl_propertylist_load_regexp(
                     cpl_frame_get_filename(first_frame),
-                                                 0, REGMANDATORY "|^ESO ", 0);
+                                                 0, REGMANDATORY
+                                                 "|^ESO |^ARCFILE$", 0);
             cpl_ensure_code(plist != NULL, cpl_error_get_code());
         }
         else {
@@ -767,10 +783,11 @@ cpl_error_code cpl_dfs_setup_product_header(cpl_propertylist *header,
                     cpl_frame_get_filename(first_frame), 0, REGMANDATORY, 0);
             cpl_ensure_code(plist != NULL, cpl_error_get_code());
             plist2 = cpl_propertylist_load_regexp(
-                    cpl_frame_get_filename(inherit_frame), 0, "^ESO", 0);
+                    cpl_frame_get_filename(inherit_frame), 0,
+                    "^ESO |^ARCFILE$", 0);
             cpl_ensure_code(plist2 != NULL, cpl_error_get_code());
             if (cpl_propertylist_copy_property_regexp(plist, plist2, 
-                                                      "^ESO", 0)) {
+                                                      "^ESO|^ARCFILE$", 0)) {
                 cpl_propertylist_delete(plist2);
                 return cpl_error_set_where_();
             }
@@ -804,9 +821,26 @@ cpl_error_code cpl_dfs_setup_product_header(cpl_propertylist *header,
         }
     }
 
+
+    /*
+     * Determine the product level from the inherited frame properties, i.e.
+     * the next free slot in the product processing history.
+     */
+
+    while (!empty) {
+        ++level;
+
+        snprintf(cval, FLEN_VALUE, PRO_RECi_ID, level);
+        empty = !cpl_propertylist_has(plist, cval);
+    }
+
+
     /*
      * Here copy all the HIERARCH.ESO._ keywords, excluding the
-     * HIERARCH.ESO.DPR._ .PRO._, .DRS._, and .QC._ keywords.
+     * HIERARCH.ESO.DPR._, .DRS._, and .QC._ keywords.
+     *
+     * HIERARCH.ESO.PRO._ keywords are also ignored with the exception of
+     * the processing history PRO.RECi._
      */
 
     /* Also copy the mandatory keywords, if any */
@@ -814,6 +848,28 @@ cpl_error_code cpl_dfs_setup_product_header(cpl_propertylist *header,
     cpl_propertylist_copy_property_regexp(header, plist,
                                           "^ESO (DPR|DRS|PRO|QC) ", 1);
 
+    /* Copy the processing history */
+
+    cpl_propertylist_copy_property_regexp(header, plist,
+                                          "^ESO PRO (REC[1-9]*) ", 0);
+
+    /* Copy or create the product's ancestor */
+
+    if (!cpl_propertylist_has(header, PRO_ANCESTOR)) {
+
+        if (cpl_propertylist_has(plist, PRO_ANCESTOR)) {
+            cpl_propertylist_copy_property(header, plist, PRO_ANCESTOR);
+        }
+        else if (cpl_propertylist_has(plist, ARCFILE)) {
+
+            const char *s = cpl_propertylist_get_string(plist, ARCFILE);
+
+            cpl_propertylist_append_string(header, PRO_ANCESTOR, s);
+
+        }
+
+    }
+
     cpl_propertylist_delete(plist);
 
 
@@ -934,8 +990,9 @@ cpl_error_code cpl_dfs_setup_product_header(cpl_propertylist *header,
      * PRO REC1 ID
      */
 
-    cpl_propertylist_update_string(header, PRO_REC_ID, recid);
-    cpl_propertylist_set_comment(header, PRO_REC_ID,
+    snprintf(cval, FLEN_VALUE, PRO_RECi_ID, level);
+    cpl_propertylist_update_string(header, cval, recid);
+    cpl_propertylist_set_comment(header, cval,
                                  "Pipeline recipe (unique) identifier");
 
 
@@ -946,18 +1003,19 @@ cpl_error_code cpl_dfs_setup_product_header(cpl_propertylist *header,
     /* cpl_propertylist_update_string(header,
                                       PRO_REC_DRS_ID, cpl_get_version()); */
 
-    cpl_propertylist_update_string(header, PRO_REC_DRS_ID,
+    snprintf(cval, FLEN_VALUE, PRO_RECi_DRS_ID, level);
+    cpl_propertylist_update_string(header, cval,
                                    PACKAGE "-" PACKAGE_VERSION);
-    cpl_propertylist_set_comment(header, PRO_REC_DRS_ID,
+    cpl_propertylist_set_comment(header, cval,
                                  "Data Reduction System identifier");
 
     /*
      * PRO REC1 PIPE ID
      */
 
-/*    snprintf(cval, FLEN_VALUE, "%s/%s", PACKAGE, PACKAGE_VERSION); */
-    cpl_propertylist_update_string(header, PRO_REC_PIPE_ID, pipeline_id);
-    cpl_propertylist_set_comment(header, PRO_REC_PIPE_ID,
+    snprintf(cval, FLEN_VALUE, PRO_RECi_PIPE_ID, level);
+    cpl_propertylist_update_string(header, cval, pipeline_id);
+    cpl_propertylist_set_comment(header, cval,
                                  "Pipeline (unique) identifier");
 
 
@@ -978,13 +1036,13 @@ cpl_error_code cpl_dfs_setup_product_header(cpl_propertylist *header,
 
             ++nraw;
 
-            snprintf(cval, FLEN_VALUE, PRO_REC_RAWi_NAME, nraw);
+            snprintf(cval, FLEN_VALUE, PRO_RECi_RAWi_NAME, level, nraw);
             cpl_propertylist_update_string(header, cval,
                      cpl_get_base_name(cpl_frame_get_filename(frame)));
             cpl_propertylist_set_comment(header, cval,
                                          "File name of raw frame");
 
-            snprintf(cval, FLEN_VALUE, PRO_REC_RAWi_CATG, nraw);
+            snprintf(cval, FLEN_VALUE, PRO_RECi_RAWi_CATG, level, nraw);
             cpl_propertylist_update_string(header, cval,
                                            cpl_frame_get_tag(frame));
             cpl_propertylist_set_comment(header, cval,
@@ -1036,19 +1094,19 @@ cpl_error_code cpl_dfs_setup_product_header(cpl_propertylist *header,
 
             ++ncal;
 
-            snprintf(cval, FLEN_VALUE, PRO_REC_CALi_NAME, ncal);
+            snprintf(cval, FLEN_VALUE, PRO_RECi_CALi_NAME, level, ncal);
             cpl_propertylist_update_string(header, cval,
                      cpl_get_base_name(cpl_frame_get_filename(frame)));
             cpl_propertylist_set_comment(header, cval,
                                          "File name of calibration frame");
 
-            snprintf(cval, FLEN_VALUE, PRO_REC_CALi_CATG, ncal);
+            snprintf(cval, FLEN_VALUE, PRO_RECi_CALi_CATG, level, ncal);
             cpl_propertylist_update_string(header, cval,
                                            cpl_frame_get_tag(frame));
             cpl_propertylist_set_comment(header, cval,
                                          "Category of calibration frame");
 
-            snprintf(cval, FLEN_VALUE, PRO_REC_CALi_DATAMD5, ncal);
+            snprintf(cval, FLEN_VALUE, PRO_RECi_CALi_DATAMD5, level, ncal);
 
             switch (cpl_is_fits(cpl_frame_get_filename(frame))) {
             case 0:
@@ -1156,11 +1214,11 @@ cpl_error_code cpl_dfs_setup_product_header(cpl_propertylist *header,
             return cpl_error_set_(CPL_ERROR_UNSPECIFIED);
         }
 
-        snprintf(cval, FLEN_VALUE, PRO_REC_PARAMi_NAME, npar);
+        snprintf(cval, FLEN_VALUE, PRO_RECi_PARAMi_NAME, level, npar);
         cpl_propertylist_update_string(header, cval, kname);
         cpl_propertylist_set_comment(header, cval, comment);
 
-        snprintf(cval, FLEN_VALUE, PRO_REC_PARAMi_VALUE, npar);
+        snprintf(cval, FLEN_VALUE, PRO_RECi_PARAMi_VALUE, level, npar);
         cpl_propertylist_update_string(header, cval, pval);
         cpl_propertylist_set_comment(header, cval, dval);
 
@@ -1226,8 +1284,350 @@ cpl_error_code cpl_dfs_update_product_header(cpl_frameset * self)
         ? cpl_error_set_where_() : CPL_ERROR_NONE;
 }
 
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief
+    Update DFS and DICB required header information of product frames.
+
+  @param set    The frameset from which the product frames are taken.
+  @param flags  Bit mask for selecting the digital signatures to be written.
+
+  @return
+   The function returns @c CPL_ERROR_NONE on success, or an appropriate CPL
+   error code otherwise.
+
+  @note
+   Each product frame must correspond to a FITS file created with a CPL
+   FITS saving function.
+
+  The function takes all frames marked as products from the input frameset
+  @em set. For each product the header information @c PIPEFILE is updated
+  unconditionally. In addition, depending on the bit mask @em flags, the
+  @c DATAMD5 data hash and/or the standard FITS checksums are computed and
+  written to the product header. If a digital signature is not selected by
+  @em flags when the function is called, its corresponding header keyword(s)
+  are removed from the product frame.
+
+   @error
+     <table class="ec" align="center">
+       <tr>
+         <td class="ecl">CPL_ERROR_NULL_INPUT</td>
+         <td class="ecr">
+            An input pointer is <tt>NULL</tt>.
+         </td>
+       </tr>
+       <tr>
+         <td class="ecl">CPL_ERROR_DATA_NOT_FOUND</td>
+         <td class="ecr">
+           The input <i>framelist</i> contains a frame of type product with a
+           missing filename.
+         </td>
+       </tr>
+       <tr>
+         <td class="ecl">CPL_ERROR_BAD_FILE_FORMAT</td>
+         <td class="ecr">
+           The input <i>framelist</i> contains a frame of type product without
+           a FITS card with key 'DATAMD5'.
+         </td>
+       </tr>
+       <tr>
+         <td class="ecl">CPL_ERROR_FILE_IO</td>
+         <td class="ecr">
+           The input <i>framelist</i> contains a frame of type product for which
+           the FITS card with key 'DATAMD5' could not be updated.
+         </td>
+       </tr>
+     </table>
+   @enderror
+
+*/
+/*----------------------------------------------------------------------------*/
+
+cpl_error_code
+cpl_dfs_sign_products(const cpl_frameset *set, unsigned int flags)
+{
+
+    cpl_ensure_code(set != NULL, CPL_ERROR_NULL_INPUT);
+
+    if (cpl_frameset_is_empty(set) || (flags == CPL_DFS_SIGNATURE_NONE)) {
+        return CPL_ERROR_NONE;
+    }
+
+
+    const cpl_frame *frame;
+
+    cpl_error_code status = CPL_ERROR_NONE;
+
+    cpl_frameset_iterator *it = cpl_frameset_iterator_new(set);
+
+    while (((frame = cpl_frameset_iterator_get_const(it)) != NULL) &&
+            (status == CPL_ERROR_NONE)) {
+
+        if (cpl_frame_get_group(frame) == CPL_FRAME_GROUP_PRODUCT) {
+            status = _cpl_dfs_sign_product(frame, flags);
+        }
+
+        cpl_errorstate _status = cpl_errorstate_get();
+
+        cpl_frameset_iterator_advance(it, 1);
+
+        if (cpl_error_get_code() == CPL_ERROR_ACCESS_OUT_OF_RANGE) {
+            cpl_errorstate_set(_status);
+        }
+
+    }
+
+    cpl_frameset_iterator_delete(it);
+
+
+    status |= cpl_io_fits_close_tid(CPL_IO_FITS_ALL);
+
+    return status ? cpl_error_set_where_() : CPL_ERROR_NONE;
+
+}
+
 /**@}*/
 
+/*----------------------------------------------------------------------------*/
+/**
+  @internal
+
+  @brief
+    Write DFS and DICB required header information to pipeline products.
+
+  @param frame  The frame to be updated.
+  @param flags  Bit mask for selecting the digital signatures to be written.
+
+  @return
+   The function returns @c CPL_ERROR_NONE on success, or an appropriate CPL
+   error code otherwise.
+
+  @see cpl_dfs_sign_products()
+ */
+/*----------------------------------------------------------------------------*/
+
+inline static cpl_error_code
+_cpl_dfs_sign_product(const cpl_frame *frame, cxuint flags)
+{
+
+    const cxchar *filename = cpl_frame_get_filename(frame);
+
+    if (filename == NULL) {
+
+        cpl_error_set(cpl_func, CPL_ERROR_DATA_NOT_FOUND);
+        return CPL_ERROR_DATA_NOT_FOUND;
+
+    }
+
+    if (cpl_is_fits(filename)) {
+
+        cxint _status = 0;
+
+        fitsfile *fptr;
+
+
+        if (cpl_io_fits_open_diskfile(&fptr, filename, READWRITE, &_status)) {
+
+            return cpl_error_set_fits(CPL_ERROR_BAD_FILE_FORMAT,
+                                      _status, fits_open_diskfile,
+                                      "filename='%s'", filename);
+
+        }
+
+
+        cxint chdu;
+        cxint nhdu;
+
+        fits_get_num_hdus(fptr, &nhdu, &_status);
+
+        if (_status) {
+
+            return cpl_error_set_fits(CPL_ERROR_FILE_IO,
+                                      _status, fits_get_num_hdus,
+                                      "filename='%s'", filename);
+
+        }
+
+
+        /*
+         * Move file pointer to the primary HDU
+         */
+
+        fits_movabs_hdu(fptr, 1, NULL, &_status);
+
+        if (_status) {
+
+            return cpl_error_set_fits(CPL_ERROR_BAD_FILE_FORMAT,
+                                      _status, fits_movabs_hdu,
+                                      "filename='%s'", filename);
+
+        }
+
+
+        /*
+         * Write primary HDU signatures
+         */
+
+        fits_update_key_str(fptr, PIPEFILE, filename,
+                            "Filename of data product", &_status);
+
+        if (_status) {
+
+            return cpl_error_set_fits(CPL_ERROR_FILE_IO,
+                                      _status, fits_update_key_str,
+                                      "filename='%s', key='"
+                                      PIPEFILE "'", filename);
+
+        }
+
+
+        if (flags & CPL_DFS_SIGNATURE_DATAMD5) {
+
+            cxchar *md5sum = cpl_dfs_find_md5sum(fptr);
+
+            if (!md5sum) {
+                return cpl_error_set_where_();
+            }
+
+            fits_update_key_str(fptr, DATAMD5, md5sum,
+                                "MD5 checksum", &_status);
+            cpl_free(md5sum);
+
+            if (_status) {
+
+                return cpl_error_set_fits(CPL_ERROR_FILE_IO,
+                                          _status, fits_update_key_str,
+                                          "filename='%s', key='"
+                                          DATAMD5 "', value='%s'",
+                                          filename, md5sum);
+
+            }
+
+        }
+        else {
+
+            fits_delete_key(fptr, DATAMD5, &_status);
+
+            if (_status == KEY_NO_EXIST) {
+                _status = 0;
+            }
+
+        }
+
+
+        if (flags & CPL_DFS_SIGNATURE_CHECKSUM) {
+
+            fits_write_chksum(fptr, &_status);
+
+            if (_status) {
+
+                return cpl_error_set_fits(CPL_ERROR_FILE_IO,
+                                          _status, fits_write_chksum,
+                                          "filename='%s'", filename);
+
+            }
+
+        }
+        else {
+
+            fits_delete_key(fptr, CHECKSUM, &_status);
+
+            if (_status == KEY_NO_EXIST) {
+                _status = 0;
+            }
+
+            fits_delete_key(fptr, DATASUM, &_status);
+
+            if (_status == KEY_NO_EXIST) {
+                _status = 0;
+            }
+
+        }
+
+
+        /*
+         * Write extension HDU signatures
+         */
+
+        if (nhdu > 1) {
+
+            /*
+             * Move file pointer to the first extension
+             */
+
+            fits_movabs_hdu(fptr, 2, NULL, &_status);
+
+            if (_status) {
+
+                return cpl_error_set_fits(CPL_ERROR_BAD_FILE_FORMAT,
+                                          _status, fits_movabs_hdu,
+                                          "filename='%s'", filename);
+
+            }
+
+            for (chdu = 2; chdu <= nhdu; ++chdu) {
+
+                if (flags & CPL_DFS_SIGNATURE_CHECKSUM) {
+
+                    fits_write_chksum(fptr, &_status);
+
+                    if (_status) {
+
+                        return cpl_error_set_fits(CPL_ERROR_FILE_IO,
+                                                  _status, fits_write_chksum,
+                                                  "filename='%s'", filename);
+
+                    }
+
+                }
+                else {
+
+                    fits_delete_key(fptr, CHECKSUM, &_status);
+
+                    if (_status == KEY_NO_EXIST) {
+                        _status = 0;
+                    }
+
+                    fits_delete_key(fptr, DATASUM, &_status);
+
+                    if (_status == KEY_NO_EXIST) {
+                        _status = 0;
+                    }
+
+                    if (_status) {
+
+                        return cpl_error_set_fits(CPL_ERROR_FILE_IO,
+                                                  _status, fits_delete_key,
+                                                  "filename='%s'", filename);
+
+                    }
+
+                }
+
+                if (chdu < nhdu) {
+                    fits_movrel_hdu(fptr, 1, NULL, &_status);
+                }
+
+            }
+
+        }
+
+
+        if (cpl_io_fits_close_file(fptr, &_status)) {
+
+            return cpl_error_set_fits(CPL_ERROR_FILE_IO,
+                                      _status, fits_close_file,
+                                      "filename='%s'", filename);
+
+        }
+
+    }
+
+    return CPL_ERROR_NONE;
+
+}
+
 
 /*----------------------------------------------------------------------------*/
 /**
