@@ -1,4 +1,4 @@
-/* $Id: cpl_detector.c,v 1.33 2012/10/29 14:27:53 llundin Exp $
+/* $Id: cpl_detector.c,v 1.33 2012-10-29 14:27:53 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2012/10/29 14:27:53 $
+ * $Date: 2012-10-29 14:27:53 $
  * $Revision: 1.33 $
- * $Name: cpl-6_3_1 $
+ * $Name: not supported by cvs2svn $
  */
 
 /*-----------------------------------------------------------------------------
@@ -39,12 +39,9 @@
 #include <cpl_math_const.h>
 #include <cpl_error_impl.h>
 
-#include <stdio.h>
-#include <stdlib.h>
 #include <string.h>
-#include <unistd.h>
 #include <math.h>
-#include <complex.h>
+#include <assert.h>
 
 /*-----------------------------------------------------------------------------
                                    Define
@@ -148,7 +145,7 @@ static cpl_bivector * cpl_bivector_gen_ring_poisson(const double *,
   @param    ron_nsamp   to specify the nb of samples (<0 to use default)
   @param    noise       Output parameter: noise in the frame.
   @param    error       Output parameter: error on the noise.
-  @return   the #_cpl_error_code_ or CPL_ERROR_NONE
+  @return   CPL_ERROR_NONE on success or the relevant #_cpl_error_code_ on error
   @see      rand()
   @note     No calls to srand() are made from CPL
 
@@ -199,7 +196,7 @@ cpl_error_code cpl_flux_get_noise_window(const cpl_image * diff,
   @param    ron_nsamp   to specify the nb of samples (<0 to use default)
   @param    bias        Output parameter: bias in the frame.
   @param    error       Output parameter: error on the bias.
-  @return   the #_cpl_error_code_ or CPL_ERROR_NONE
+  @return   CPL_ERROR_NONE on success or the relevant #_cpl_error_code_ on error
   @see      cpl_flux_get_noise_window(), rand()
   @note     No calls to srand() are made from CPL
  */
@@ -322,9 +319,9 @@ static cpl_error_code cpl_flux_get_window(
   @param    zone_def    Zone where the readout noise is to be computed.
   @param    ron_hsize   to specify half size of squares (<0 to use default)
   @param    ron_nsamp   to specify the nb of samples (<0 to use default)
-  @param    noise       Output parameter: noise in the frame.
-  @param    error       Output parameter: error on the noise.
-  @return   the #_cpl_error_code_ or CPL_ERROR_NONE
+  @param    noise       On success, *noise is the noise in the frame.
+  @param    error       NULL, or on success, *error is the error in the noise
+  @return   CPL_ERROR_NONE on success or the relevant #_cpl_error_code_ on error
   @see      cpl_flux_get_noise_window(), rand()
   @note     No calls to srand() are made from CPL
 
@@ -350,9 +347,8 @@ cpl_error_code cpl_flux_get_noise_ring(
         double          *   error)
 {
     cpl_bivector    *   sample_reg;
-    double          *   px;
-    double          *   py;
-    cpl_size            zone[4];
+    const double    *   px;
+    const double    *   py;
     const cpl_size      hsize = ron_hsize < 0 ? RING_RON_HS      : ron_hsize;
     const cpl_size      nsamp = ron_nsamp < 0 ? RING_RON_SAMPLES : ron_nsamp;
     cpl_vector      *   rms_list;
@@ -374,17 +370,20 @@ cpl_error_code cpl_flux_get_noise_ring(
     /* position and it would bias the measurement. */
     sample_reg = cpl_bivector_gen_ring_poisson(zone_def, nsamp+1, nsamp+1);
     cpl_ensure_code(sample_reg != NULL, CPL_ERROR_ILLEGAL_INPUT);
-    px = cpl_bivector_get_x_data(sample_reg);
-    py = cpl_bivector_get_y_data(sample_reg);
+    px = cpl_bivector_get_x_data_const(sample_reg);
+    py = cpl_bivector_get_y_data_const(sample_reg);
 
     /* Now, for each valid sample, extract a vignette and compute the RMS */
     rms_list = cpl_vector_new(nsamp + 1);
-    cpl_ensure_code(rms_list, CPL_ERROR_NULL_INPUT);
+    assert( rms_list != NULL );
     rsize = 0;
     for (i=0; i < nsamp + 1; i++) {
+        cpl_errorstate prestate = cpl_errorstate_get();
+        cpl_size       zone[4];
         /* Add 0.5 to round off to nearest (positive) integer */
-        const double dx = 0.5 + zone_def[0] + px[i] * cos(py[i]);
-        const double dy = 0.5 + zone_def[1] + px[i] * sin(py[i]);
+        const double   dx = 0.5 + zone_def[0] + px[i] * cos(py[i]);
+        const double   dy = 0.5 + zone_def[1] + px[i] * sin(py[i]);
+        double stdev;
 
         zone[0] = (cpl_size)dx - hsize;
         if (zone[0] < 1) continue;
@@ -398,11 +397,16 @@ cpl_error_code cpl_flux_get_noise_ring(
         zone[3] = (cpl_size)dy + hsize;
         if (zone[3] > ny) continue;
 
-        cpl_vector_set(rms_list, rsize, cpl_image_get_stdev_window(diff,
-                                                                   zone[0],
-                                                                   zone[2],
-                                                                   zone[1],
-                                                                   zone[3]));
+        stdev = cpl_image_get_stdev_window(diff, zone[0], zone[2],
+                                           zone[1], zone[3]);
+
+        if (!cpl_errorstate_is_equal(prestate)) {
+            /* All pixels could be bad */
+            cpl_errorstate_set(prestate);
+            continue;
+        }
+
+        cpl_vector_set(rms_list, rsize, stdev);
         rsize++;
         /* At this point rms_list has rsize elements */
     }
@@ -701,7 +705,7 @@ static cpl_bivector * cpl_bivector_gen_ring_poisson(
 {
     double              min_dist;
     cpl_size            gnp;
-    cpl_bivector    *   list;
+    cpl_bivector    *   list = cpl_bivector_new(np);
     double              cand_r, cand_t;
     cpl_boolean         ok;
     cpl_size            start_ndx;
@@ -709,17 +713,12 @@ static cpl_bivector * cpl_bivector_gen_ring_poisson(
     cpl_size            i;
     /* Corrected Homogeneity factor */
     const cpl_size      homogc = 0 < homog && homog < np ? homog : np;
-    double          *   px;
-    double          *   py;
+    double          *   px = cpl_bivector_get_x_data(list);
+    double          *   py = cpl_bivector_get_y_data(list);
 
     /* Check entries */
-    cpl_ensure(r, CPL_ERROR_NULL_INPUT, NULL);
-    cpl_ensure(np > 0, CPL_ERROR_ILLEGAL_INPUT, NULL);
-
-    list = cpl_bivector_new(np);
-    cpl_ensure(list, CPL_ERROR_NULL_INPUT,NULL);
-    px = cpl_bivector_get_x_data(list);
-    py = cpl_bivector_get_y_data(list);
+    cpl_ensure(r    != NULL, CPL_ERROR_NULL_INPUT,    NULL);
+    cpl_ensure(list != NULL, CPL_ERROR_ILLEGAL_INPUT, NULL);
 
     r1 = (cpl_size)r[2];
     r2 = (cpl_size)r[3];
