@@ -848,9 +848,8 @@ cpl_error_code cpl_wlcalib_find_best_1d(cpl_polynomial       * self,
                                         const cpl_polynomial * guess,
                                         const cpl_vector     * spectrum,
                                         void                 * model,
-                                        cpl_error_code      (* filler)
-                                        (cpl_vector *, void *,
-                                         const cpl_polynomial *),
+                                        cpl_error_code      (* filler)(cpl_vector *,
+                                                void *, const cpl_polynomial *),
                                         const cpl_vector     * wl_search,
                                         cpl_size               nsamples,
                                         cpl_size               hsize,
