@@ -1,4 +1,4 @@
-/* $Id: cpl_gasgano.c,v 1.23 2013/01/30 15:59:18 rpalsa Exp $
+/* $Id: cpl_gasgano.c,v 1.23 2013-01-30 15:59:18 rpalsa Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2006 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: rpalsa $
- * $Date: 2013/01/30 15:59:18 $
+ * $Date: 2013-01-30 15:59:18 $
  * $Revision: 1.23 $
- * $Name: cpl-6_3_1 $
+ * $Name: not supported by cvs2svn $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -1559,15 +1559,22 @@ static jobjectArray makeFrameArray(JNIEnv *env, cpl_frameset *fset) {
    /* Populate the array with new Frame objects constructed from the frames
     * in the frameset. */
    ifrm = 0;
-   for (frame = cpl_frameset_get_first(fset); ok && frame;
-        frame = cpl_frameset_get_next(fset)) {
+
+   cpl_frameset_iterator *it = cpl_frameset_iterator_new(fset);
+
+   while (ok && (frame = cpl_frameset_iterator_get(it)))
+   {
       ok = ok &&
          (jFrame = makeJavaFrame(env, frame));
       if (ok) {
          (*env)->SetObjectArrayElement(env, jFrames, ifrm++, jFrame);
       }
+
+      cpl_frameset_iterator_advance(it, 1);
    }
 
+   cpl_frameset_iterator_delete(it);
+
    /* Return the array. */
    return ok ? jFrames : NULL;
 }
